﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace ELTE.Forms.Sudoku
{
    public partial class GameForm : Form
    {
        private Int32 _StepCount; // lépések száma
        private Int32[,] _Numbers; // számok a táblában
        private Button[,] _Table; // gombok a táblában

        public GameForm()
        {
            InitializeComponent();
           
            GenerateTable();
            StartGame();
        }

        private void GenerateTable() // új tábla létrehozása
        {
            _Numbers = new Int32[9, 9]; // mátrixok generálása
            _Table = new Button[9, 9];
            for (Int32 i = 0; i < _Table.GetLength(0); i++)
                for (Int32 j = 0; j < _Table.GetLength(1); j++)
                {
                    _Table[i, j] = new Button();
                    _Table[i, j].Location = new Point(5 + 25 * i, 25 + 25 * j); // elhelyezkedés
                    _Table[i, j].Size = new Size(25, 25); // méret
                    _Table[i, j].TabIndex = i * 9 + j; // a gomb számát a TabIndex-ben tároljuk
                    _Table[i, j].FlatStyle = FlatStyle.Flat; // lapított stípus
                    _Table[i, j].MouseClick += new MouseEventHandler(GameForm_MouseClick);
                    // közös eseménykezelő hozzárendellése minden gombhoz

                    Controls.Add(_Table[i, j]);
                    // felevesszük az ablakra a gombot
                }
        }
        private void StartGame() // új játék kezdése
        {
            for (Int32 i = 0; i < _Numbers.GetLength(0); i++)
                for (Int32 j = 0; j < _Numbers.GetLength(1); j++)
                {
                    _Numbers[i, j] = 0; // éretékek kinullázása
                    _Table[i, j].Text = String.Empty;
                    _Table[i, j].Enabled = true; // gomb bekapcsolása
                    _Table[i, j].BackColor = Color.White; // háttérszín fehér
                }
            _StepCount = 0;
            _LabelStepCount.Text = _StepCount.ToString();
        }
        private void LoadGame(String fileName) // játék betöltése
        {
            StreamReader reader = new StreamReader(fileName);
            for (Int32 i = 0; i < _Numbers.GetLength(0); i++)
            {
                String[] numbers = reader.ReadLine().Split(' '); // beolvasunk egy sort, és a szóköz mentén széttöredezzük
                for (Int32 j = 0; j < _Numbers.GetLength(1); j++)
                {
                    _Numbers[i, j] = Int32.Parse(numbers[j]);
                    if (_Numbers[i, j] == 0) // ha nincs kitöltve a mező
                    {
                        _Table[i, j].Text = String.Empty;
                        _Table[i, j].Enabled = true;
                        _Table[i, j].BackColor = Color.White;
                    }
                    else // ha ki van töltve
                    {
                        _Table[i, j].Text = _Numbers[i, j].ToString();
                        _Table[i, j].Enabled = false; // gomb kikapcsolása
                        _Table[i, j].BackColor = Color.Yellow; // háttérszín sárga
                    }
                }
            }
            reader.Close();

            _StepCount = 0;
            _LabelStepCount.Text = _StepCount.ToString();
        }
        private void SaveGame(String fileName) // játék mentése
        {
            StreamWriter writer = new StreamWriter(fileName);
            for (Int32 i = 0; i < _Numbers.GetLength(0); i++)
            {
                for (Int32 j = 0; j < _Numbers.GetLength(1); j++)
                    writer.Write(_Numbers[i, j] + " ");
                writer.WriteLine();
            }
            writer.Close();
        }
        private Boolean CheckTable(Int32 x, Int32 y) // érték ellenőrzése a kitöltésnél
        {
            if (_Numbers[x, y] == 0)
                return true;
            else
            {
                Boolean good = true;
                // sor ellenőrzése:
                for (Int32 i = 0; i < 9; i++)
                    if (_Numbers[i, y] == _Numbers[x, y] && x != i)
                        good = false;
                // oszlop ellenőrzése:
                for (Int32 j = 0; j < 9; j++)
                    if (_Numbers[x, j] == _Numbers[x, y] && y != j)
                        good = false;
                // ház ellenőrzése:
                Int32 startX, startY, endX, endY;
                if (x < 3)
                {
                    startX = 0; endX = 3;
                }
                else if (x < 6)
                {
                    startX = 3; endX = 6;
                }
                else
                {
                    startX = 6; endX = 9;
                }
                if (y < 3)
                {
                    startY = 0; endY = 3;
                }
                else if (y < 6)
                {
                    startY = 3; endY = 6;
                }
                else
                {
                    startY = 6; endY = 9;
                }
                for (Int32 i = startX; i < endX; i++)
                    for (Int32 j = startY; j < endY; j++)
                    {
                        if (_Numbers[i, j] == _Numbers[x, y] && x != i && y != j)
                            good = false;
                    }
                return good;
            }
        }
        private Boolean CheckGame() // játék végének ellenőrzése
        {
            Int32 fieldsLeft = 0;

            for (Int32 i = 0; i < _Numbers.GetLength(0); i++)
                for (Int32 j = 0; j < _Numbers.GetLength(1); j++)
                    if (_Numbers[i, j] == 0)
                        fieldsLeft++;
            return (fieldsLeft == 0);
        }

        private void GameForm_MouseClick(object sender, MouseEventArgs e)
        {
            // a TabIndex-ből megkapjuk a sort és oszlopot:
            Int32 x = ((Button)sender).TabIndex / 9;
            Int32 y = ((Button)sender).TabIndex % 9;
            
            do
            {
                _Numbers[x, y] = (_Numbers[x, y] + 1) % 10;
                // következő szám
            }
            while (!CheckTable(x, y)); // amíg nem jó az érték

            if (_Numbers[x, y] == 0)
                _Table[x, y].Text = String.Empty;
            else
                _Table[x, y].Text = _Numbers[x, y].ToString();
            _StepCount++; // lépésszám növelés
            _LabelStepCount.Text = _StepCount.ToString();

            if (CheckGame()) // ha vége a játéknak
            {
                MessageBox.Show("Gratulálok!" + Environment.NewLine + "Összesen " + _StepCount + " lépést tettél.", "Játék vége!", MessageBoxButtons.OK, MessageBoxIcon.Information);
                StartGame();
            }
        }

        // menü eseménykezelői:
        private void MenuFileNewGame_Click(object sender, EventArgs e)
        {
            StartGame();
        }
        private void MenuFileLoadGame_Click(object sender, EventArgs e)
        {
            if (_OpenFileDialog.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    LoadGame(_OpenFileDialog.FileName);
                }
                catch (IOException)
                {
                    MessageBox.Show("Játék betöltése sikertelen!" + Environment.NewLine + "Hibás az elérési út, vagy a fájlformátum.", "Hiba!", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    StartGame();
                }
            }
        }
        private void MenuFileSaveGame_Click(object sender, EventArgs e)
        {
            if (_SaveFileDialog.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    SaveGame(_SaveFileDialog.FileName);
                }
                catch (IOException)
                {
                    MessageBox.Show("Játék mentése sikertelen!" + Environment.NewLine + "Hibás az elérési út, vagy a könyvtár nem írható.", "Hiba!", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }
        private void MenuFileExit_Click(object sender, EventArgs e)
        {
            Close();
        }
    }
}
